#!/usr/bin/perl
#
# Copyright 2012 SIXNET, LLC
#
#  Notice: This document contains proprietary, confidential information
#  owned by SIXNET, LLC and is protected as an unpublished work under
#  the Copyright Law of the United States. No part of this document may
#  be copied, disclosed to others or used for any purpose without the
#  written permission of Digitronics Inventioneering Corporation.
#

#
# Package with locking functions
#

package Sixnet::SVM::Locks;

require strict;
require warnings;
require Exporter;

use Sixnet::SVM::Debug;

use Fcntl qw(:DEFAULT :flock);

our @ISA = qw(Exporter);
our @EXPORT = qw(openLock lock unlock);

##
# Open a lock file - create it if it does not exist.
#
# @param name - name to give to lock file
#
# @return file handle on success
# @return not defined on failure;
#
sub openLock($)
{
    my ($name) = @_;
    my $APP;
    my $LCK;
    my $FLCK;

    if (defined($name)) {

        $APP = "/var/lock/$name";
        $LCK = $APP . ".lockfile";
        if (!sysopen($FLCK, $LCK, O_RDONLY | O_CREAT)) {
            debugPrint("Failed to open lock file: $LCK\n");
        }
    }
    return $FLCK;
}

##
# Obtain a file lock
#
# @param FLCK - file handle to lock file
# @param opts - flag containing option for lock.
#
# The options must consist of the following:
#               LOCK_EX for exclusive locking.
#               LOCK_SH for shared locking.
#
# It may also have the LOCK_NB set if the caller does not want
# to block waiting for the lock.
#
# @return 0 if success
# @return -1 if undefined handle
# @return -2 if invalid lock options.
# @return -3 if failed to unlock the lock file.
#
sub lock($$)
{
    my ($FLCK, $opts) = @_;
    my $retVal = 0;

    if (!defined($FLCK)) {
        debugPrint("Undefined lock handle\n");
        $retVal = -1;
    }
    elsif (($opts & ~(LOCK_EX | LOCK_SH | LOCK_NB)) ||
           (($opts & (LOCK_EX | LOCK_SH)) == (LOCK_EX | LOCK_SH)) ||
           (($opts & (LOCK_EX | LOCK_SH)) == 0)) {
        #
        # Check if the options passed in are ones that we expect.
        #  (Only LOCK_EX or LOCK_SH, with possibly LOCK_NB set)
        #
        debugPrint("Invalid options: $opts\n");
        $retVal = -2;
    }
    elsif (!flock($FLCK, $opts)) {
        debugPrint("Failed to get lock file: $LCK\n");
        $retVal = -3;
    }

    return $retVal;
}

##
# Release a file lock
#
# @param FLCK - file handle to lock file
#
# @return 0 if success
# @return -1 if undefined handle
# @return -2 if failed to unlock the lock file.
#
sub unlock($)
{
    my ($FLCK) = @_;
    my $retVal = 0;
    my $opts = LOCK_UN;

    if (!defined($FLCK)) {
        debugPrint("Undefined lock handle\n");
        $retVal = -1;
    }
    elsif (!flock($FLCK, $opts)) {
        debugPrint("Failed to unlock file\n");
        $retVal = -2;
    }
    return $retVal;
}


sub Init()
{
    return 1;
}

return Init();
