#!/bin/sh
#
# Monitor the load and running processes on the
# system, and output data to a set of rotated log files.
# If logs already exist, they are bundled up and saved
# until a defined limit is reached.
#
# This is normally started after system boot and initialization
# is completed.  It runs until SIGTERM is received.
#

LOGDIR=$1

if [ -z "$LOGDIR" ] ; then
    echo "Usage: $0 /path/to/logdir" 2>&1
    exit 1
fi

if ! cd $LOGDIR ; then
    echo "LOGDIR $LOGDIR not found" 2>&1
    exit 1
fi

# save logs for this many restarts minus 1, max 9
SAVELOGS=4

# sleep (seconds) between each poll of running processes
SLEEPTIME=5

# number of logs to save, max 1000
NUMLOGS=30

LOGTEMPLATE=loadmon

#
# if any logs exist, bundle them for possible
# review later.
#
OLDLOGS=`find . -name $LOGTEMPLATE.\*.log`
if [ "$OLDLOGS" ] ; then
    #
    # rotate logs if needed
    #
    if [ -e $LOGTEMPLATE.$SAVELOGS.tar.gz ] ; then
        rm $LOGTEMPLATE.$SAVELOGS.tar.gz
    fi
    logs=`ls -1 $LOGTEMPLATE.[0-9].tar.gz 2>/dev/null | sort -r`
    for sl in $logs ; do
        n=`echo $sl | sed -e "s/$LOGTEMPLATE\.\([0-9]\)\.tar\.gz/\1/"`
        mv $sl $LOGTEMPLATE.`expr $n + 1`.tar.gz
    done

    # bundle current
    tar cf $LOGTEMPLATE.0.tar $LOGTEMPLATE.*.log
    gzip $LOGTEMPLATE.0.tar && rm $LOGTEMPLATE.*.log
fi

#
# loop until killed, logging load and process info
#
# NOTE: these logs rotate round-robin, to avoid a lot of I/O
# to reorder the files continuously
#
n=0
while true ; do
    (
        uptime
        ps -A -opid,user,pcpu,cputime,cmd --sort=-pcpu
    ) > $LOGTEMPLATE.`printf "%03d" $n`.log
    n=`expr \( $n + 1 \) % $NUMLOGS`

    sleep $SLEEPTIME
done
